#!/bin/sh
#version 1.1
set -u

PRODUCT=CPE
PRODUCT_LOWCASE=cpe
upgrade_path=/tmp/upgrade
version_path=/etc/"$PRODUCT_LOWCASE"agent/
conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf
bak_conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf.upgradebak

logfile="/var/log/update.log"
target="version"
patch_target_type=""
target_name=""
result_id=""
upgrade_rpm=""

ret_success=0
ret_success_restart=1
ret_success_reboot=2
ret_fail_roolback=6

log()
{
    echo `date` "upgrade:" $@ >> $logfile
}

#write_upgrade_result_conf $1=success/failure
write_upgrade_result_conf()
{
    rm $bak_conf_file -f
    cp $conf_file $bak_conf_file -f
    if [ $? -eq 0 ];then
        sed -i "/upgraderesult/d" $bak_conf_file
        sed -i "/upgradeID/d" $bak_conf_file
        echo "upgraderesult=$1"  >> $bak_conf_file
        echo "upgradeID=${result_id}"  >> $bak_conf_file
        cp $bak_conf_file $conf_file -f
    fi
}

#upgrade_product_version $1=rpm_name
upgrade_product_version()
{
    /sbin/sysupgrade -v $1
}

upgrade_version()
{
    log "----------------$(date)--------------------"
    log "Version upgrade, target=${target_name} result_id=${result_id}"

    if [ ! -f ${upgrade_path}/${target_name} ];then
        log "Could't find $target_name in directory ${upgrade_path}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
        exit
    fi

    write_upgrade_result_conf "failure"

    upgrade_product_version "${upgrade_path}/${target_name}"
}

manual_version_upgrade()
{
    log "----------------$(date)--------------------"
    log "Manual Version upgrade"

    upgrade_rpm=`ls /tmp/upgrade | grep "tar.gz" | grep -v PATCH`
    [ -z "$upgrade_rpm" ] && {
        log "UPGRADE version failed, could't find upgrade target"
        exit 1
    }

    upgrade_product_version "${upgrade_path}/${upgrade_rpm}"
    if [[ $? -ne 0 ]];then
        log "UPGRADE version failed"
    fi
}

#handel_patch_return $1=ret
handel_patch_return()
{
    log "Upgrade ${target_name} upgradeId ${result_id} success"
    echo ${target_name##*-} > "${version_path}/PATCH-${patch_target_type}-VERSION"
    local value=$1
    if [ $value -eq 0 ];then
        cpeagent upgradeSuccess
    elif [ $value -eq ${ret_success_restart} ];then
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        /etc/init.d/cpectl restart
    elif [ $value -eq ${ret_success_reboot} ];then
        write_upgrade_result_conf "success"
        log "reboot device"
        reboot
    fi
}

#upgrade_product_patch $1=target script
upgrade_product_patch()
{
    local script=$1
    log "install ${target_name} start"
    sh ${script} install >> $logfile 2>&1
    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]]; then
        log "install success $ret"
    else
        log "install failed $ret"
    fi
    return $ret
}

#upgrade_product_patch_rollback $1=target script
patch_rollback()
{
    local script=$1
    log "UPGRADE patch failed, try ROLLBACK"
    log "uninstall ${target_name}"
    sh ${script} uninstall >> $logfile 2>&1
}

#is_patch_target $1=target_name $2=THIRD/CUSTOM/DRIVER
is_patch_target()
{
    echo "$1" | grep -q "$2"
    return $?
}

get_patch_target_type()
{
    is_patch_target $target_name "THIRD"
    if [ $? -eq 0 ];then
        patch_target_type="THIRD"
        return 0
    fi
    
    is_patch_target $target_name "CUSTOM"
    if [ $? -eq 0 ];then
        patch_target_type="CUSTOM"
        return 0
    fi
    
    is_patch_target $target_name "DRIVER"
    if [ $? -eq 0 ];then
        patch_target_type="DRIVER"
        return 0
    fi
}

upgrade_patch()
{
    log "----------------$(date)--------------------"
    log "Patch upgrade, target=${target_name} result_id=${result_id}"

    get_patch_target_type
    if [ -z $patch_target_type ];then
        log "Invalid patch, exit"
        cpeagent upgradeFailed
        exit
    fi

    if [[ ! -d ${upgrade_path}/${target_name} || ! -f ${upgrade_path}/${target_name}/${target_name}.sh ]];then
        log "Could't find $target_name in directory ${upgrade_path}/${target_name}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
    fi

    if [ ! -x "${upgrade_path}/${target_name}/${target_name}.sh" ];then
        chmod 751 ${upgrade_path}/${target_name}/${target_name}.sh
    fi

    upgrade_product_patch "${upgrade_path}/${target_name}/${target_name}.sh"

    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]];then
        handel_patch_return $ret
    else
        log "Upgrade ${target_name} upgradeId ${result_id} failed"
        cpeagent upgradeFailed
        if [ $ret == $ret_fail_roolback ];then
            patch_rollback "${upgrade_path}/${target_name}/${target_name}.sh"
        fi
    fi
}

help()
{
    echo "Usage: $0 --target=<patch/version> --target_name=CPE-3.5.1-20211112.x86_64.rpm --result_id=xxxx"
    echo "      --target/-t         : Target  patch/version"
    echo "      --target_name/-tn   : Upgrade target name, e.g CPE-3.5.1-20211112.x86_64.rpm"
    echo "      --result_id/-r      : upgrade Id"
}

for opt in ${@};
do
    str="="
    result=$(echo $opt | grep "$str")
    if [ -n $result ]; then
        k=${opt%=*}
        v=${opt#*=}
    else
        k=${opt}
    fi

    case ${k} in
        "--target"|"-t") target=${v};;
        "--target_name"|"-tn") target_name=${v};;
        "--result_id"|"-r") result_id=${v};;
        *) echo "${opt} is error";;
    esac
done

if [ $# -lt 3 ];then
    target="manual_version"
else
    [[ -z $target || -z $target_name || -z $result_id ]] && {
        help
        exit 1
    }
fi

if [ ${target} == "patch" ]; then
    upgrade_patch
elif [ ${target} == "version" ]; then
    upgrade_version
elif [ ${target} == "manual_version" ]; then
    echo "manual_version"
    manual_version_upgrade
else
    log "----------------$(date)--------------------"
    log "Unsupported target ${target} result_id=${result_id}"
    cpeagent upgradeFailed
fi
