#!/bin/sh
#v1.0.7
#2020.11.11
#gloabal variable, edit carefully
gVersion=1.1.0
#default check interval, second
gInterval=60

#default check timeout, second
gTimeout=3
script_name=$(basename $0)
is_exist=`ps | grep $script_name | grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ $# -ne 0 ];then
    if [ $1 == "EnableDelay" ];then
        sleep 120
    fi
fi

is_3_3_version=`cat /proc/net/lw-conf/lan-tbl | grep Statistics | head -n1` > /dev/null
if [ -z "$is_3_3_version" ];then
    wanGateWay=`cat /proc/net/lw-conf/wan-tbl | awk 'NR>1{print $5}' | grep -v "0.0.0.0"`
    lanGateWay=`cat /proc/net/lw-conf/lan-tbl | awk 'NR>1{print $4}' | grep -v "0.0.0.0"`
else
    wanGateWay=`cat /proc/net/lw-conf/wan-tbl | grep GatewayIp | grep -v "0.0.0.0" | awk -F "GatewayIp: " '{print $2}'`
    lanGateWay=`cat /proc/net/lw-conf/lan-tbl |grep NextHop | grep -v "0.0.0.0" | awk -F "NextHop: " '{print $2}'`
fi

#ping
gPingTarget="
$wanGateWay
$lanGateWay
"

gPingTargetCount=`echo ${gPingTarget} | awk -F ' ' '{print NF}'`
gPingOptions="-i 0.3"

gPingTimeout=$gTimeout
gPingCount=6

#wget
#Note : since wget on arm/mips doest not support option --no-check-certificate, it may have issue to deal with https
gWgetTarget="

"
gWgetTimeout=$gTimeout
gWgetTargetCount=`echo ${gWgetTarget} | awk -F ' ' '{print NF}'`

#log
gPingLogPath=/var/log/monitor/detect_ping/
gWgetLogPath=/var/log/monitor/detect_wget/

#log check days >= 1
gLogCheckDay=7

#log check interval
gLogCheckInterval=60

checkLogFile()
{
    find $gPingLogPath -type f -mtime +$gLogCheckDay -exec rm -- '{}' \; 
    find $gWgetLogPath -type f -mtime +$gLogCheckDay -exec rm -- '{}' \; 
}

pinglog()
{
    [ -f "$gPingLogFile" ] && echo "$(date +'%Y%m%d-%H:%M:%S') $@" >> $gPingLogFile
}
    
pingnotimelog()
{
    [ -f "$gPingLogFile" ] && echo "$@" >> $gPingLogFile
}

wgetlog()
{
    [ -f "$gWgetLogFile" ] && echo "$(date +'%Y%m%d-%H:%M:%S') $@" >> $gWgetLogFile
}
wgetnotimelog()
{
    [ -f "$gWgetLogFile" ] && echo "$@" >> $gWgetLogFile
}

initConfig()
{
    is_mips=`uname -a | grep "mips"` > /dev/null
    [ ! -z "is_mips" ] && {
        gPingOptions=""
    }
    
    [ -z "$gInterval" -o $gInterval -eq 0  ] && {
        gInterval=60
    }

    [ -z "$gTimeout" -o $gTimeout -eq 0  ] && {
        gTimeout=3
    }

    [ -z "$gPingCount" -o $gPingCount -eq 0  ] && {
        gPingCount=6
    }

    [ -z "$gPingTimeout" -o $gPingTimeout -eq 0  ] && {
        gPingTimeout=$gTimeout
    }

    [ -z "$gWgetTimeout" -o $gWgetTimeout -eq 0  ] && {
        gWgetTimeout=$gTimeout
    }

    if [ ${gLogCheckDay} -lt 1 ];then
        gLogCheckDay=0
    else
        gLogCheckDay=`expr ${gLogCheckDay} - 1`
    fi

    [ -z "$gPingLogPath" ] && {
        gPingLogPath=/var/log/monitor/detect_ping/
    }
    mkdir -p $gPingLogPath 2>/dev/null

    [ -z "$gWgetLogPath" ] && {
        gWgetLogPath=/var/log/monitor/detect_wget/
    }
    mkdir -p $gWgetLogPath 2>/dev/null
    updateLogFileName
}

updateLogFileName()
{
    local logPrefix

    logPrefix=$(date +'%Y%m%d')

    gPingLogFileName=ping-${logPrefix}.log
    gPingLogFile=$gPingLogPath/$gPingLogFileName
    [ -f "$gPingLogFile" ] || touch $gPingLogFile

    gWgetLogFileName=wget-${logPrefix}.log
    gWgetLogFile=$gWgetLogPath/$gWgetLogFileName
    [ -f "$gWgetLogFile" ] || touch $gWgetLogFile
}

dumpCfg()
{
    pingnotimelog "configuration:"
    pingnotimelog "Ping: count = $gPingCount timeout = $gPingTimeout"
    pingnotimelog "    target: $gPingTarget"
    pingnotimelog "log: file = $gPingLogFile"
    pingnotimelog ""

    wgetnotimelog "configuration:"
    wgetnotimelog "Wget: timeout = $gWgetTimeout"
    wgetnotimelog "    target: $gWgetTarget"
    wgetnotimelog "log: file = $gWgetLogFile"
    wgetnotimelog ""
}


pingDetect()
{
    local target=$1 
    local result 

    [ -z "$target" ] && return

    result=$(ping ${gPingOptions} -q -c $gPingCount -w $gPingTimeout ${target} | grep "packet loss" | sed 's/ /\n/g' | grep % | tr -cd "[0-9]")
    if [ $result -ne 0 ];then
        pinglog "Ping $target: packet loss ${result}%"
    fi
}

tcpingDetect()
{
    local target=$1
    local port=$2
    local result

    [ -z "$target" ] && return
    [ -z "$port" ] && return 

    result=$(tcping -t $gTcpingTimeout $target $port 2>&1)
    tcpinglog "${result}"
}

wgetgDetect()
{
    local target=$1

    [ -z "$target" ] && return

    result=$(wget -T $gWgetTimeout $target -O /dev/null 2>&1 |grep -vE '[0-9]s|^$|\.\.\.\.')
    wgetlog "wget $target"
    wgetnotimelog "$result"
    wgetnotimelog ""
}

gProgFile=$(readlink -f $0)
gProg=$(basename $0)

initConfig

pinglog "${gProg}-$gVersion start running ..."
wgetlog "${gProg}-$gVersion start running ..."
dumpCfg

count=0
wgetnewline=0
while true
do
    updateLogFileName
    [ $count -gt 0 -a $(expr $count % $gLogCheckInterval) -eq 0 ] && {
        checkLogFile
    }

    [ $wgetnewline -eq 1 ] && {
        wgetnotimelog ""
        wgetnotimelog ""
    }

    for i in $(seq 1 ${gPingTargetCount});do
        target=`echo ${gPingTarget} | awk '{print $'$i'}'`
        pingDetect $target &
    done

    wgetnewline=0
    for i in $(seq 1 ${gWgetTargetCount});do
        target=`echo ${gWgetTarget} | awk '{print $'$i'}'`
        wgetnewline=1
        wgetgDetect $target
    done

    count=$(expr $count + 1)
    sleep $gInterval
done
