#!/bin/sh
#v2.0
#2021.07.29
Monitor_Path=/var/log/monitor/pppoe_monitor
script_name=$(basename $0)

pppoe_phyif=""
pppoe_pppname=""
ping_target="
61.135.169.121
111.206.176.91
47.95.164.112"

start_flag=0
stop_flag=0
sleep_time=10
max_pcap_num=50
enable_delay=0
ping_target_count=`echo ${ping_target} | awk -F ' ' '{print NF}'`

help() {
    echo "Usage: $0 <Start/Stop> [SetInterval  <Time>] [EnableDelay]"
    echo "      e.g. $0 Start SetInterval 10 EnableDelay &"
    echo "NOTE: Interval in unit of minute. Default time is 10 second if no interval is specified."
    echo "      EnableDelay: wait 120 second to collect info. Default is disable"
}

stop_script(){
    for pid in `ps | grep "$script_name Start" | grep -v grep | awk '{print $1}'`
    do
        kill -9 $pid
    done
}

#ping_test pppoe_pppname ping_target
ping_test(){
    ping -I $1 -c 5 -w 5 $2 > /dev/null 2>&1
    if [ $? -ne 0 ];then                         
        let $3=$3+1
    fi
}

get_value(){              
	newname=${1}${2}           
	eval "echo $`echo $newname`"
}             

pref=last_trigger_time_
                            
pre_set(){
	for i in $(seq 1 10); do    
		eval "$pref$i=0"        
		val=`get_value $pref $i`     
	done  
	
	pppoe_list=`cat /proc/net/lw-conf/if-tbl | grep "Name: ppp" | cut -d ":" -f 2 | sed 's/ //g'` > /dev/null
	pppoe_num=`cat /proc/net/lw-conf/if-tbl | grep "Name: ppp" | awk '{print $1}'` > /dev/null
	pppoe_list_count=`echo ${pppoe_list} | awk -F ' ' '{print NF}'`
	pppoe_count=0

	for j in $(seq 1 ${pppoe_list_count});do
		num=`echo ${pppoe_num} | awk '{print $'$j'}'`
		pppoe=`echo ${pppoe_list} | awk '{print $'$j'}'`
		state=`cat /proc/net/lw-conf/if-tbl | grep  "${num}" | grep "State: 0"`> /dev/null
		if [ -n "$state" ];then
			pppoe_pppname="$pppoe_pppname ${pppoe}"
			pppoe_phyifnum=`cat /proc/net/lw-conf/if-tbl | grep "PPPoEIf: ${pppoe}" | awk '{print $1}'`
			pppoe_phyif_tmp=`cat /proc/net/lw-conf/if-tbl | grep "${pppoe_phyifnum}" | grep "Name:" | cut -d ":" -f 2 | sed 's/ //g'`
			pppoe_phyif="$pppoe_phyif $pppoe_phyif_tmp"
			pppoe_count=$((${pppoe_count}+1))
		fi
	done
}

#ppp_restart pppoe_phyif
ppp_restart(){
    for pid in `ps | grep "pppd call" | grep "$1" | awk '{print $1}' | grep -v grep`
    do
        kill -9 $pid
    done
    pppd call $1 & >> ${log_file}
    sleep 60
}

[[ $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        sleep_time=$1
    fi
    if [ $1 == "EnableDelay" ];then
        enable_delay=1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    help
    exit 1
fi

is_exist=`ps | grep ${script_name}| grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ ${enable_delay} -eq 1 ];then
    sleep 120
fi

if [ ! -d ${Monitor_Path} ];then
    mkdir -p $Monitor_Path
fi

pre_set

while true
do
    log_file=${Monitor_Path}/pppoe_monitor_`date "+%Y-%m-%d"`.log 
    pcap_file_num=`find ${Monitor_Path} -type f -name "*_pppoe_monitor.pcap" | wc -l`
    if [ $pcap_file_num -gt $max_pcap_num ];then
        pcap_del_file_num=`expr $pcap_file_num - $max_pcap_num`
        find ${Monitor_Path} -type f -name "*_pppoe_monitor.pcap" | xargs ls -dta | tail -n ${pcap_del_file_num} | xargs rm -rf
    fi
    
    find ${Monitor_Path} -type f -name "pppoe_monitor_*" -mtime +7 -exec rm -- '{}' \;
    
    time_now=`date +%s`
    for i in $(seq 1 ${pppoe_count});do
        fail_times=0
        pppoe_phyif_target=`echo ${pppoe_phyif} | awk '{print $'$i'}'`
        exist=`ps | grep pppd | grep $pppoe_phyif_target | grep -v grep` > /dev/null
        if [ -n "${exist}" ];then
            echo "`date "+%Y-%m-%d_%H%M"` ${pppoe_phyif_target} present" >> ${log_file}
        else
            echo "`date "+%Y-%m-%d_%H%M"` pppd process for ${pppoe_phyif_target} absent, restart pppd call" >> ${log_file}
            ppp_restart ${pppoe_phyif_target}
        fi
		
        pppoe_pppname_target=`echo ${pppoe_pppname} | awk '{print $'$i'}'`
        for j in $(seq 1 ${ping_target_count});do
		    target=`echo ${ping_target} | awk '{print $'$j'}'`
            ping_test ${pppoe_pppname_target} ${target} fail_times
        done
		
        val=`get_value $pref $i`
        if [ $fail_times -eq ${ping_target_count} ];then
            if [ $time_now -gt $val ];then
                val=$(($val+600))
                eval "$pref$i=$val"
                tcpdump -i $pppoe_phyif_target -w ${Monitor_Path}/${pppoe_phyif_target}_`date "+%Y-%m-%d_%H%M"`_pppoe_monitor.pcap > /dev/null 2>&1 &
                sleep 5 
                kill -9 `ps | grep "tcpdump -i $pppoe_phyif_target" | grep -v grep | awk '{print $1}'` > /dev/null 2>&1
            fi
            echo "`date "+%Y-%m-%d_%H%M"` ${pppoe_phyif_target} ping test fialed ,restart pppd call" >> ${log_file}
            ppp_restart ${pppoe_phyif_target}
        fi
    done
    sleep $sleep_time
done
