#!/bin/sh

####################################################################
# create dynamic route(zebra/bgpd/ospfd/ospf6d) config file for VPN namespace
# $1: VPN-ID
####################################################################

VPNID=$1
[ -z "$VPNID" ] && {
    logger -t "createDynRtConf" "Error: must set VPN-ID(${VPNID})"
    exit 1
}

VPN_PREFIX="ns${VPNID}_"
CONF_DIR=/etc/quagga

ZEBRA_CONF=${CONF_DIR}/${VPN_PREFIX}zebra.conf
BGPD_CONF=${CONF_DIR}/${VPN_PREFIX}bgpd.conf
OSPFD_CONF=${CONF_DIR}/${VPN_PREFIX}ospfd.conf
OSPF6D_CONF=${CONF_DIR}/${VPN_PREFIX}ospf6d.conf

cat > ${ZEBRA_CONF} <<EOF
log file /var/log/quagga/quagga.log
log vpn-id $VPNID

debug zebra events
debug zebra rib
debug zebra kernel
debug zebra rib queue

table 231
EOF

cat > ${BGPD_CONF} <<EOF
log file /var/log/quagga/quagga.log
log vpn-id $VPNID

debug bgp
debug bgp events
debug bgp zebra
debug bgp updates in
debug bgp updates out
debug bgp fsm
EOF

cat > ${OSPFD_CONF} <<EOF
log file /var/log/quagga/quagga.log
log vpn-id $VPNID

debug ospf nsm
debug ospf zebra
debug ospf packet dd
debug ospf packet ls-ack
debug ospf packet ls-request
debug ospf packet ls-update
debug ospf lsa flooding
debug ospf lsa install
EOF

cat > ${OSPF6D_CONF} <<EOF
log file /var/log/quagga/quagga.log
log vpn-id $VPNID

debug ospf6 flooding
debug ospf6 message dbdesc
debug ospf6 message lsack
debug ospf6 message lsreq
debug ospf6 message lsupdate
debug ospf6 message unknown
debug ospf6 lsa router
debug ospf6 lsa network
debug ospf6 lsa link
debug ospf6 lsa inter-prefix
debug ospf6 lsa inter-router
debug ospf6 lsa as-external
debug ospf6 lsa intra-prefix
debug ospf6 lsa unknown
debug ospf6 neighbor
debug ospf6 route table
debug ospf6 zebra
debug ospf6 border-routers
debug ospf6 abr
debug ospf6 asbr
debug ospf6 interface
debug ospf6 spf process
EOF

exit 0
